/* Copyright (c) 2016 Raffaele Arecchi
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package aktrs.test.local;

import java.util.ArrayList;
import java.util.List;

import aktrs.rt.Actor;

public class ScatterActor extends Actor {
	private List<Actor> workers = new ArrayList<Actor>();
	private int roundRobinCurrentIndex = 0;

	@Override
	protected void onMessage(Object message, Actor sender) {
		if (message instanceof StartScatterGather) {
			StartScatterGather m = (StartScatterGather) message;
			for (int i = 1; i <= m.numberoOfWorkers; i++) {
				workers.add(getSystem().summon("/" + getSystem().id + "/worker" + i, WorkerActor.class));
			}
			Actor gathererActor = getSystem().summon("/" + getSystem().id + "/gatherer", GatherActor.class);
			for (Actor actor : workers) {
				tell(actor, GathererAssignment.create(gathererActor.getAddress()));
			}
			tell(gathererActor, message);
		} else if (message instanceof CalculateFibonacciMessage) {
			CalculateFibonacciMessage m = (CalculateFibonacciMessage) message;
			roundRobinCurrentIndex++;
			if (roundRobinCurrentIndex > (workers.size() - 1)) {
				roundRobinCurrentIndex = 0;
			}
			tell(workers.get(roundRobinCurrentIndex), m);
		}
	}
}
