/* Copyright (c) 2016 Raffaele Arecchi
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package aktrs.test.local;

import java.math.BigInteger;
import java.util.logging.Level;

import aktrs.messages.AverageProcessingTimeByClassRequest;
import aktrs.messages.AverageProcessingTimeByClassResponse;
import aktrs.messages.ProfilingMode;
import aktrs.messages.ToggleProfilingMessage;
import aktrs.rt.Actor;
import aktrs.rt.ActorSystem;

/**
 * Profiling actor test
 * 
 * @author Raffaele Arecchi
 *
 */

public class Test3 {

	public static void main(String[] args) {
		ActorSystem sys = new ActorSystem("fib");
		sys.setLogLevel(Level.FINE);

		Actor supervisor = sys.summon("/fib/fibonacciSupervisor", Supervisor.class);
		Actor fib1 = sys.summon("/fib/fibonacciCalc1", FibonacciActor.class);
		supervisor.tell(fib1, ToggleProfilingMessage.create(ProfilingMode.ON));
		supervisor.tell(fib1, CalculateFibonacciMessage.create(new BigInteger("38")));
		supervisor.tell(fib1, CalculateFibonacciMessage.create(new BigInteger("39")));
		AverageProcessingTimeByClassRequest profilingmessage = new AverageProcessingTimeByClassRequest();
		profilingmessage.klass = CalculateFibonacciMessage.class;
		profilingmessage.replyTo = supervisor.getAddress();
		supervisor.tell(fib1, profilingmessage);
	}

	public static class Supervisor extends Actor {

		public Supervisor() {
		}

		@Override
		protected void onMessage(Object message, Actor sender) {
			if (message instanceof AverageProcessingTimeByClassResponse) {
				AverageProcessingTimeByClassResponse m = (AverageProcessingTimeByClassResponse) message;
				System.out.println("Average processing time for klass " + m.klass + " is " + m.averageProcessingTime);
				getSystem().shutdown("end of profiling");
			}
		}
	}

}
