/* Copyright (c) 2016 Raffaele Arecchi
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package aktrs.test.remote;

import java.io.Serializable;
import java.math.BigInteger;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.logging.Level;

import aktrs.rt.Actor;
import aktrs.rt.ActorAddress;
import aktrs.rt.ActorSystem;
import aktrs.test.local.CalculateFibonacciMessage;
import aktrs.test.local.ResultFibonacciMessage;
import aktrs.test.local.SlowFibonacci;

public class Test1_Remote1 {

	public static void main(String[] args) throws UnknownHostException {
		ActorSystem system = new ActorSystem("requester-sys", 9001);
		system.setLogLevel(Level.FINE);
		system.link("calculator-sys", InetAddress.getLocalHost(), 9002);

		Actor calculator = system.summon("/calculator-sys/calculator", CalculatorActor.class);
		Actor requesterActor = system.summon("/requester-sys/requester", RequesterActor.class);
		system.evoke("/requester-sys/system").tell(requesterActor, AssignCalculator.create(calculator.getAddress()));
		system.evoke("/requester-sys/system").tell(requesterActor, CalculateFibonacciMessage.create(new BigInteger("39")));
	}

	public static class RequesterActor extends Actor {
		private Actor calculator;

		public RequesterActor() {
		}

		@Override
		protected void onMessage(Object message, Actor sender) {
			if (message instanceof ResultFibonacciMessage) {
				ResultFibonacciMessage m = (ResultFibonacciMessage) message;
				System.out.println("Actor " + address.toString() + " received fibonacci computation as " + m.number);
				getSystem().shutdown("finished");
			} else if (message instanceof AssignCalculator) {
				calculator = getSystem().evoke(((AssignCalculator) message).next);
			} else if (message instanceof CalculateFibonacciMessage) {
				tell(calculator, message);
			}
		}
	}

	public static class CalculatorActor extends Actor {

		public CalculatorActor() {
		}

		@Override
		protected void onMessage(Object message, Actor sender) {
			if (message instanceof CalculateFibonacciMessage) {
				CalculateFibonacciMessage m = (CalculateFibonacciMessage) message;
				System.out.println("Actor " + m.number + " received fibonacci request of computation for " + m.number);
				tell(sender, ResultFibonacciMessage.create(SlowFibonacci.nthFibonacci(m.number)));
			}
		}
	}

	public static class AssignCalculator implements Serializable {
		private static final long serialVersionUID = 1L;
		public ActorAddress next;

		public AssignCalculator() {
		}

		public static AssignCalculator create(ActorAddress next) {
			AssignCalculator m = new AssignCalculator();
			m.next = next;
			return m;
		}
	}

}
